Package: bsseq
Version: 1.47.1
Encoding: UTF-8
Title: Analyze, manage and store whole-genome methylation data
Description: A collection of tools for analyzing and visualizing whole-genome
    methylation data from sequencing. This includes whole-genome bisulfite
    sequencing and Oxford nanopore data.
Authors@R: c(person(c("Kasper", "Daniel"), "Hansen", role = c("aut", "cre"),
    email = "kasperdanielhansen@gmail.com", comment = c(ORCID = "0000-0003-0086-0687")),
    person("Peter", "Hickey", role = "aut", email = "peter.hickey@gmail.com", comment = c(ORCID = "0000-0002-8153-6258")),
    person("Hervé", "Pagès", role = "ctb"),
    person("Aaron", "Lun", role = "ctb"))
Depends: R (>= 4.0), methods, BiocGenerics, GenomicRanges (>= 1.41.5),
        SummarizedExperiment (>= 1.19.5)
Imports: IRanges (>= 2.23.9), Seqinfo, scales, stats, parallel, tools,
        graphics, Biobase, locfit, gtools, data.table (>= 1.11.8),
        S4Vectors (>= 0.27.12), R.utils (>= 2.0.0), DelayedMatrixStats
        (>= 1.5.2), permute, limma, DelayedArray (>= 0.15.16), Rcpp,
        BiocParallel, BSgenome, Biostrings, utils, HDF5Array (>=
        1.19.11), rhdf5, beachmat (>= 2.23.2)
Suggests: testthat, bsseqData, BiocStyle, rmarkdown, knitr, Matrix,
        doParallel, rtracklayer, BSgenome.Hsapiens.UCSC.hg38,
        batchtools
Collate: utils.R hasGRanges.R BSseq-class.R BSseqTstat_class.R
        BSseq_utils.R combine.R read.bismark.R read.bedMethyl.R
        read.modbam2bed.R read.modkit.R BSmooth.R BSmooth.tstat.R
        dmrFinder.R gof_stats.R plotting.R fisher.R permutations.R
        BSmooth.fstat.R BSseqStat_class.R getStats.R hdf5_utils.R
        DelayedArray_utils.R collapseBSseq.R FWIRanges-class.R
        FWGRanges-class.R findLoci.R Likelihood_functions.R
License: Artistic-2.0
VignetteBuilder: knitr
URL: https://github.com/kasperdanielhansen/bsseq
BugReports: https://github.com/kasperdanielhansen/bsseq/issues
biocViews: DNAMethylation
LinkingTo: Rcpp, beachmat, assorthead (>= 1.1.4)
SystemRequirements: C++17
NeedsCompilation: yes
RoxygenNote: 7.1.0
git_url: https://git.bioconductor.org/packages/bsseq
git_branch: devel
git_last_commit: 46b4295
git_last_commit_date: 2025-10-31
Repository: Bioconductor 3.23
Date/Publication: 2025-11-04
Packaged: 2025-11-04 21:34:55 UTC; biocbuild
Author: Kasper Daniel Hansen [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-0086-0687>),
  Peter Hickey [aut] (ORCID: <https://orcid.org/0000-0002-8153-6258>),
  Hervé Pagès [ctb],
  Aaron Lun [ctb]
Maintainer: Kasper Daniel Hansen <kasperdanielhansen@gmail.com>
