# R package: bsseq

[![Bioc release status](http://www.bioconductor.org/shields/build/release/bioc/bsseq.svg)](https://bioconductor.org/checkResults/release/bioc-LATEST/bsseq)
[![Bioc devel status](http://www.bioconductor.org/shields/build/devel/bioc/bsseq.svg)](https://bioconductor.org/checkResults/devel/bioc-LATEST/bsseq)
[![Bioc downloads rank](https://bioconductor.org/shields/downloads/release/bsseq.svg)](http://bioconductor.org/packages/stats/bioc/bsseq/)
[![Bioc support](https://bioconductor.org/shields/posts/bsseq.svg)](https://support.bioconductor.org/tag/bsseq)
[![Bioc history](https://bioconductor.org/shields/years-in-bioc/bsseq.svg)](https://bioconductor.org/packages/release/bioc/html/bsseq.html#since)
[![Bioc last commit](https://bioconductor.org/shields/lastcommit/devel/bioc/bsseq.svg)](http://bioconductor.org/checkResults/devel/bioc-LATEST/bsseq/)
[![Bioc dependencies](https://bioconductor.org/shields/dependencies/release/bsseq.svg)](https://bioconductor.org/packages/release/bioc/html/bsseq.html#since)

This is the developer version of Bioconductor package [bsseq](http://bioconductor.org/packages/devel/bioc/html/bsseq.html).  Install in R as:

```r
install.packages("BiocManager")
BiocManager::install("bsseq", version = "devel")
```




