\name{Internals}
\alias{matrixOrNULL-class}
\title{Internals}
\description{
   The \code{matrixOrNULL} class is a class union, representing a slot
   which is either a matrix or \code{NULL} (representing missing
   information). 
}
\value{
  Undocumented
}
\keyword{internal}
